﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class Patron
    Private ActiveID As Long
    Private StartingOver As Boolean
    Private AllowPatronRecord As Boolean

    Public Overrides Function AddRecord() As Long
        ' ----- Add a new record.
        ActiveID = -1&
        AllowPatronRecord = False
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Function CanUserAdd() As Boolean
        ' ----- Check the user for security access: add.
        Return SecurityProfile(LibrarySecurity.ManagePatrons)
    End Function

    Public Overrides Function CanUserEdit() As Boolean
        ' ----- Check the user for security access: edit.
        Return SecurityProfile(LibrarySecurity.ManagePatrons)
    End Function

    Public Overrides Function CanUserDelete() As Boolean
        ' ----- Check the user for security access: delete.
        Return SecurityProfile(LibrarySecurity.ManagePatrons)
    End Function

    Public Overrides Function DeleteRecord(recordID As Long) As Boolean
        ' ----- The user wants to delete the records.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim transInfo As LibraryTransaction = Nothing

        ' ----- Confirm with the user.
        If (MessageBox.Show("Do you really wish to delete the patron?",
            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
            DialogResult.Yes) Then Return False

        ' ----- Make sure this record is not in use.
        sqlText = "SELECT COUNT(*) FROM PatronCopy WHERE Patron = @RecordID " &
            "AND Returned = 0 AND Missing = 0"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                MessageBox.Show("You cannot delete this record because " &
                    "the patron still has items checked out.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        Catch ex As Exception
            GeneralError("Patron.DeleteRecord.Check", ex)
            Return False
        End Try

        ' ----- Delete the records.
        Try
            transInfo = TransactionBegin()

            sqlText = "DELETE FROM PatronCopy WHERE Patron = @Record"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            sqlText = "DELETE FROM Patron WHERE ID = @Record"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", recordID)
            ExecuteSQL(sqlRun, transInfo)

            TransactionCommit(transInfo)
        Catch ex As Exception
            TransactionRollback(transInfo)
            GeneralError("Patron.DeleteRecord.Delete", ex)
            Return False
        End Try
        Return True
    End Function

    Public Overrides Function EditRecord(recordID As Long) As Long
        ' ----- Edit an existing record. NOTE: If you modify this routine,
        '       you must also modify the EditRecordLimited routine.
        ActiveID = recordID
        AllowPatronRecord = True
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Function EditRecordLimited(recordID As Long) As Long
        ' ----- Edit an existing record. NOTE: If you modify this routine,
        '       you must also modify the EditRecord routine. The Limited
        '       version does not allow access to the PatronRecord form.
        ActiveID = recordID
        AllowPatronRecord = False
        PrepareFormFields()
        Me.ShowDialog()
        If (Me.DialogResult = DialogResult.OK) Then _
            Return ActiveID Else Return -1&
    End Function

    Public Overrides Sub FillListWithRecords(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items. Prompt the
        '       user first.
        destList.Items.Clear()
        SearchForRecord(destList, exceededMatches)
    End Sub

    Public Overrides Function FormatRecordName(recordID As Long) As String
        ' ----- Given an ID, return the name.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow
        Dim nameText As String

        ' ----- Initialize.
        nameText = ""

        ' ----- Retrieve the value from the database.
        sqlText = "SELECT LastName, FirstName, Active FROM Patron WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", recordID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Patron.FormatRecordName", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            nameText = DBGetText(oneRow!LastName) & ", " & DBGetText(oneRow!FirstName)
            If (DBGetBool(oneRow!Active) = False) Then nameText &= " (Inactive)"
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
        Return nameText
    End Function

    Public Overrides Function GetEditDescription() As String
        ' ----- Return the list form display title.
        Return "Use this form to manage the patrons who have check-out " &
            "privileges at the library."
    End Function

    Public Overrides Function GetEditTitle() As String
        ' ----- Return the list form display title.
        Return "Patrons"
    End Function

    Private Sub PrepareFormFields()
        ' ----- Fill in the data.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim oneRow As DataRow

        ' ----- Fill in the list of patron groups, if needed.
        If (RecordGroup.Items.Count = 0) Then
            RecordGroup.Items.Add(New ListItemData("<Not Selected>", -1&))
            sqlText = "SELECT * FROM CodePatronGroup ORDER BY FullName"
            Try
                dbInfo = CreateDataTable(sqlText)
            Catch ex As Exception
                GeneralError("Patron.PrepareFormFields.Groups", ex)
                Return
            End Try
            For Each dbRow As DataRow In dbInfo.Rows
                RecordGroup.Items.Add(New ListItemData(
                    DBGetText(dbRow!FullName), DBGetLong(dbRow!ID)))
            Next dbRow
            dbInfo.Dispose()
        End If

        ' ----- Clear the existing data.
        StartingOver = True
        RecordLastName.Text = ""
        RecordFirstName.Text = ""
        RecordActive.Checked = True
        RecordBarcode.Text = ""
        RecordPassword.Text = ""
        RecordConfirm.Text = ""
        RecordPhone.Text = ""
        RecordEmail.Text = ""
        RecordAddress.Text = ""
        RecordCity.Text = ""
        RecordState.Text = ""
        RecordPostal.Text = ""
        RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1&)
        RecordComments.Text = ""
        RecordMessage.Text = ""
        RecordLastActivity.Text = "Not Available"
        TabPatron.SelectedTab = TabGeneral
        ActItems.Visible = False
        If (ActiveID = -1&) Then Return

        ' ----- Load in the values stored in the database.
        sqlText = "SELECT * FROM Patron WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", ActiveID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Patron.PrepareFormFields.Retrieve", ex)
            Return
        End Try
        If (dbInfo.Rows.Count > 0) Then
            oneRow = dbInfo.Rows(0)
            RecordLastName.Text = DBGetText(oneRow!LastName)
            RecordFirstName.Text = DBGetText(oneRow!FirstName)
            RecordActive.Checked = DBGetBool(oneRow!Active)
            RecordBarcode.Text = DBGetText(oneRow!Barcode)
            RecordPhone.Text = DBGetText(oneRow!Phone)
            RecordEmail.Text = DBGetText(oneRow!Email)
            RecordAddress.Text = DBGetText(oneRow!Address)
            RecordCity.Text = DBGetText(oneRow!City)
            RecordState.Text = DBGetText(oneRow!State)
            RecordPostal.Text = DBGetText(oneRow!Postal)
            If (IsDBNull(oneRow!PatronGroup) = False) Then
                RecordGroup.SelectedIndex =
                    RecordGroup.Items.IndexOf(DBGetLong(oneRow!PatronGroup))
                If (RecordGroup.SelectedIndex = -1) Then _
                    RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1&)
            End If
            RecordComments.Text = DBGetText(oneRow!Comments)
            RecordMessage.Text = DBGetText(oneRow!AdminMessage)
            If (IsDBNull(oneRow!LastActivity) = False) Then _
                RecordLastActivity.Text = DBGetDate(
                oneRow!LastActivity).ToString("MMMM d, yyyy")
            ActItems.Visible = AllowPatronRecord
        End If
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Sub SearchForRecord(destList As ListBox, ByRef exceededMatches As Boolean)
        ' ----- Given a list, fill it with all defined items.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim limitText As String
        Dim nameText As String
        Dim resultCount As Integer
        Dim limitLast As String
        Dim limitFirst As String

        ' ----- Prompt the user for the limited item title.
        exceededMatches = False
        limitText = (New PatronLimit).PromptUser()

        ' ----- Use the limit to prepare the selection criteria.
        limitLast = limitText.GetSubStr(",", 1).Trim()
        limitFirst = limitText.GetSubStr(",", 2).Trim()
        If (limitLast.Length = 0) And (limitFirst.Length = 0) Then Return
        If (limitLast.Contains("*") = False) Then limitLast &= "*"
        If (limitFirst.Contains("*") = False) Then limitFirst &= "*"
        limitLast = limitLast.Replace("*", "%")
        limitFirst = limitFirst.Replace("*", "%")

        ' ----- Clear any existing values.
        destList.Items.Clear()
        resultCount = 0

        ' ----- Retrieve the values from the database.
        sqlText = "SELECT ID, LastName, FirstName, Active FROM Patron " &
            "WHERE LastName LIKE @TestLast " &
            "AND FirstName LIKE @TestFirst " &
            "ORDER BY LastName, FirstName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLast", limitLast)
        sqlRun.AddText("TestFirst", limitFirst)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            GeneralError("Patron.SearchForRecord", ex)
            Return
        End Try

        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add one item.
            resultCount += 1
            If (resultCount > SearchMatchLimit) Then
                exceededMatches = True
                Exit For
            End If
            nameText = DBGetText(dbRow!LastName) & ", " & DBGetText(dbRow!FirstName)
            If (DBGetBool(dbRow!Active) = False) Then nameText &= " (Inactive)"
            destList.Items.Add(New ListItemData(nameText, DBGetLong(dbRow!ID)))
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
    End Sub

    Public Overrides Function UsesSearch() As Boolean
        ' ----- Patron records are searchable.
        Return True
    End Function

    Private Function SaveFormData() As Boolean
        ' ----- Save the record to the database.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim newID As Long

        Me.Cursor = Cursors.WaitCursor

        ' ----- Save the data.
        If (ActiveID = -1&) Then
            ' ----- Create a new entry.
            sqlText = "INSERT INTO Patron (LastName, FirstName, Active, Comments, " &
                "AdminMessage, Barcode, [Password], Phone, Email, Address, City, " &
                "State, Postal, PatronGroup) OUTPUT INSERTED.ID VALUES (@NewLast, " &
                "@NewFirst, @NewActive, @NewComments, @NewAdmin, @NewBarcode, " &
                "@NewPassword, @NewPhone, @NewEmail, @NewAddress, @NewCity, " &
                "@NewState, @NewPostal, @NewGroup)"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewLast", RecordLastName.Text.Trim())
            sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim())
            sqlRun.AddBool("NewActive", RecordActive.Checked)
            sqlRun.AddText("NewComments", RecordComments.Text.Trim())
            sqlRun.AddText("NewAdmin", RecordMessage.Text.Trim())
            sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim())
            sqlRun.AddText("NewPassword", EncryptPassword("patron",
                RecordPassword.Text.Trim()))
            sqlRun.AddText("NewPhone", RecordPhone.Text.Trim())
            sqlRun.AddText("NewEmail", RecordEmail.Text.Trim())
            sqlRun.AddText("NewAddress", RecordAddress.Text.Trim())
            sqlRun.AddText("NewCity", RecordCity.Text.Trim())
            sqlRun.AddText("NewState", RecordState.Text.Trim())
            sqlRun.AddText("NewPostal", RecordPostal.Text.Trim())
            sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup))
            Try
                newID = DBGetLong(ExecuteSQLReturn(sqlRun))
            Catch ex As Exception
                GeneralError("Patron.SaveFormData.Add", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
            ActiveID = newID
        Else
            ' ----- Update the existing entry.
            sqlText = "UPDATE Patron SET LastName = @NewLast, " &
                "FirstName = @NewFirst, Active = @NewActive, " &
                "Comments = @NewComments, AdminMessage = @NewAdmin, " &
                "Barcode = @NewBarcode, Phone = @NewPhone, " &
                "Email = @NewEmail, Address = @NewAddress, " &
                "City = @NewCity, State = @NewState, " &
                "Postal = @NewPostal, PatronGroup = @NewGroup"
            If (RecordPassword.Text.Trim().Length <> 0) Then _
                sqlText &= ", [Password] = @NewPassword"
            sqlText &= " WHERE ID = @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("NewLast", RecordLastName.Text.Trim())
            sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim())
            sqlRun.AddBool("NewActive", RecordActive.Checked)
            sqlRun.AddText("NewComments", RecordComments.Text.Trim())
            sqlRun.AddText("NewAdmin", RecordMessage.Text.Trim())
            sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim())
            sqlRun.AddText("NewPhone", RecordPhone.Text.Trim())
            sqlRun.AddText("NewEmail", RecordEmail.Text.Trim())
            sqlRun.AddText("NewAddress", RecordAddress.Text.Trim())
            sqlRun.AddText("NewCity", RecordCity.Text.Trim())
            sqlRun.AddText("NewState", RecordState.Text.Trim())
            sqlRun.AddText("NewPostal", RecordPostal.Text.Trim())
            sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup))
            If (RecordPassword.Text.Trim().Length <> 0) Then _
                sqlRun.AddText("NewPassword", EncryptPassword("patron",
                RecordPassword.Text.Trim()))
            sqlRun.AddLong("RecordID", ActiveID)
            Try
                ExecuteSQL(sqlRun)
            Catch ex As Exception
                GeneralError("Patron.SaveFormData.Update", ex)
                Return False
            Finally
                Me.Cursor = Cursors.Default
            End Try
        End If

        ' ----- Success.
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Function ValidateFormData() As Boolean
        ' ----- Make sure the user supplied valid data. Return True on success.
        Dim sqlText As String
        Dim sqlRun As DataCommand

        ' ----- Last name is a required field.
        If (RecordLastName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the last name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordLastName.Focus()
            Return False
        End If

        ' ----- First name is a required field.
        If (RecordFirstName.Text.Trim().Length = 0) Then
            MessageBox.Show("Please supply the first name.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            RecordFirstName.Focus()
            Return False
        End If

        ' ----- Barcode, if supplied, must be numeric.
        If (RecordBarcode.Text.Trim().Length <> 0) Then
            If (DigitsOnly(RecordBarcode.Text) <> RecordBarcode.Text.Trim()) Then
                MessageBox.Show("Barcode, if supplied, must be numeric.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                TabPatron.SelectedTab = TabGeneral
                RecordBarcode.Focus()
                Return False
            End If
        End If

        ' ----- Make sure the barcode is not already in use.
        If (RecordBarcode.Text.Trim().Length <> 0) Then
            sqlText = "SELECT COUNT(*) FROM Patron WHERE UPPER(Barcode) = @TestBarcode"
			If (ActiveID <> -1&) Then sqlText &= " AND ID <> @RecordID"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddText("TestBarcode", RecordBarcode.Text.Trim().ToUpper())
            If (ActiveID <> -1&) Then _
                sqlRun.AddLong("RecordID", ActiveID)
            Try
                If (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0) Then
                    ' ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the barcode'" &
                        RecordBarcode.Text.Trim() & "'.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    TabPatron.SelectedTab = TabGeneral
                    RecordBarcode.Focus()
                    Return False
                End If
            Catch ex As Exception
                GeneralError("Patron.ValidateFormData", ex)
                Return False
            End Try
        End If

        ' ----- Password and confirm must match.
        If (RecordPassword.Text <> RecordConfirm.Text) Then
            MessageBox.Show("The Password and Confirm fields must match.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            TabPatron.SelectedTab = TabGeneral
            RecordPassword.Focus()
            Return False
        End If

        ' ----- For new patrons, a password is required.
        If (ActiveID = -1&) And (RecordPassword.Text.Trim().Length = 0) Then
            MessageBox.Show("You must supply a password for new patron records. " &
                "For an initial password, consider using the last four digits " &
                "of the patron's phone number.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            TabPatron.SelectedTab = TabGeneral
            RecordPassword.Focus()
            Return False
        End If

        ' ----- Check the matching patrons, just in case.
        If (ActiveID = -1&) Then
            RefreshMatchingPatrons()
            If (ShowMatches.ImageIndex = MatchPresent) Then
                If (MessageBox.Show("The name you supplied is similar to one or more names " &
                    "already in the database. Do you still want to save this record?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) <>
                    DialogResult.Yes) Then Return False
            End If
        End If

        ' ----- Success.
        Return True
    End Function

    Private Sub ActOK_Click(sender As Object, e As EventArgs) Handles ActOK.Click
        ' ----- The user wants to save changes.
        If (ValidateFormData() = False) Then Return
        If (SaveFormData() = False) Then Return
        Me.DialogResult = DialogResult.OK
    End Sub

    Private Sub Patron_Activated(sender As Object, e As EventArgs) Handles MyBase.Activated
        ' ----- Return the focus to the main field.
        If (StartingOver) Then RecordLastName.Focus()
        StartingOver = False
    End Sub

    Private Sub RefreshMatchingPatrons()
        ' ----- See if there are name matches for this patron.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim holdText As String
        Dim matches As Integer

        ' ----- Only performed for new records.
        If (ActiveID <> -1&) Then Return

        ' ----- Reset the display.
        ShowMatches.Text = "      No name matches"
        ShowMatches.ImageIndex = MatchNone

        ' ----- Ignore if we have insufficient data.
        If (RecordLastName.Text.Trim().Length = 0) Then Return
        If (RecordFirstName.Text.Trim().Length = 0) Then Return

        ' ----- Retrieve a count of matching records.
        sqlText = "SELECT COUNT(*) FROM Patron WHERE LastName LIKE @TestLast"
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlText &= " AND FirstName LIKE @TestFirst"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLast", RecordLastName.Text.Trim())
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlRun.AddText("TestFirst", RecordFirstName.Text.Trim().Substring(0, 1) & "%")
        Try
            matches = DBGetInteger(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            GeneralError("Patron.RefreshMatchingPatrons", ex)
            matches = 0
        End Try
        If (matches > 0) Then
            ' ----- There are matches.
            If (matches = 1) Then
                holdText = "1 match"
            Else
                holdText = matches & " matches"
            End If
            holdText &= ". Click for details."
            ShowMatches.Text = "      " & holdText
            ShowMatches.ImageIndex = MatchPresent
        End If
    End Sub

    Private Sub RecordLastName_Enter(sender As Object, e As EventArgs) Handles RecordLastName.Enter
        ' ----- Highlight the entire text.
        RecordLastName.SelectAll()
    End Sub

    Private Sub RecordFirstName_Enter(sender As Object, e As EventArgs) Handles RecordFirstName.Enter
        ' ----- Highlight the entire text.
        RecordFirstName.SelectAll()
    End Sub

    Private Sub RecordBarcode_Enter(sender As Object, e As EventArgs) Handles RecordBarcode.Enter
        ' ----- Highlight the entire text.
        RecordBarcode.SelectAll()

        ' ----- Don't allow an enter to close the form.
        Me.AcceptButton = Nothing
    End Sub

    Private Sub RecordPassword_Enter(sender As Object, e As EventArgs) Handles RecordPassword.Enter
        ' ----- Highlight the entire text.
        RecordPassword.SelectAll()
    End Sub

    Private Sub RecordConfirm_Enter(sender As Object, e As EventArgs) Handles RecordConfirm.Enter
        ' ----- Highlight the entire text.
        RecordConfirm.SelectAll()
    End Sub

    Private Sub RecordPhone_Enter(sender As Object, e As EventArgs) Handles RecordPhone.Enter
        ' ----- Highlight the entire text.
        RecordPhone.SelectAll()
    End Sub

    Private Sub RecordEmail_Enter(sender As Object, e As EventArgs) Handles RecordEmail.Enter
        ' ----- Highlight the entire text.
        RecordEmail.SelectAll()
    End Sub

    Private Sub RecordAddress_Enter(sender As Object, e As EventArgs) Handles RecordAddress.Enter
        ' ----- Highlight the entire text.
        RecordAddress.SelectAll()
    End Sub

    Private Sub RecordCity_Enter(sender As Object, e As EventArgs) Handles RecordCity.Enter
        ' ----- Highlight the entire text.
        RecordCity.SelectAll()
    End Sub

    Private Sub RecordState_Enter(sender As Object, e As EventArgs) Handles RecordState.Enter
        ' ----- Highlight the entire text.
        RecordState.SelectAll()
    End Sub

    Private Sub RecordPostal_Enter(sender As Object, e As EventArgs) Handles RecordPostal.Enter
        ' ----- Highlight the entire text.
        RecordPostal.SelectAll()
    End Sub

    Private Sub RecordLastName_Leave(sender As Object, e As EventArgs) Handles RecordLastName.Leave
        ' ----- Update the matching display.
        RefreshMatchingPatrons()
    End Sub

    Private Sub RecordFirstName_Leave(sender As Object, e As EventArgs) Handles RecordFirstName.Leave
        ' ----- Update the matching display.
        RefreshMatchingPatrons()
    End Sub

    Private Sub ShowMatches_Click(sender As Object, e As EventArgs) Handles ShowMatches.Click
        ' ----- The user wants more information on the matches.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim holdText As String
        Dim nameText As String
        Dim message As String
        Dim matches As Integer
        Dim counter As Integer

        ' ----- Only performed for new records.
        If (ActiveID <> -1&) Then Return

        ' ----- Ignore if there are no matches.
        If (ShowMatches.ImageIndex = MatchNone) Then
            MessageBox.Show("No matches.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return
        End If

        ' ----- Retrieve a count of matching records.
        matches = CInt(Val(ShowMatches.Text))
        Me.Cursor = Cursors.WaitCursor

        ' ----- Retrieve the matching records.
        sqlText = "SELECT * FROM Patron WHERE LastName LIKE @TestLast"
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlText &= " AND FirstName LIKE @TestFirst"
        sqlText &= " ORDER BY LastName, FirstName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddText("TestLast", RecordLastName.Text.Trim())
        If (RecordFirstName.Text.Trim().Length <> 0) Then _
            sqlRun.AddText("TestFirst", RecordFirstName.Text.Trim().Substring(0, 1) & "%")
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("Author.ShowMatches_Click", ex)
            Return
        End Try

        counter = 0
        holdText = ""
        For Each dbRow As DataRow In dbInfo.Rows
            ' ----- Add this author.
            nameText = DBGetText(dbRow!LastName) & ", " & DBGetText(dbRow!FirstName)
            If (DBGetBool(dbRow!Active) = False) Then nameText &= " (Inactive)"
            holdText &= vbCrLf & vbTab & nameText

            ' ----- Limit to ten matches.
            counter += 1
            If (counter >= 10) Then Exit For
        Next dbRow
        dbInfo.Dispose()
        dbInfo = Nothing
        Me.Cursor = Cursors.Default

        ' ----- Format and display the message.
        message = "The database contains " & matches.ToString() & " match"
        If (matches > 1) Then message &= "es"
        message &= " for the name:" & vbCrLf & vbCrLf & vbTab &
            RecordLastName.Text.Trim()
        If (RecordFirstName.Text.Trim().Length <> 0) Then message &= ", " &
            RecordFirstName.Text.Trim()
        If (matches > 10) Then
            message &= vbCrLf & vbCrLf & "The first ten matches are:"
        Else
            message &= vbCrLf & vbCrLf & "The matches are:"
        End If
        message &= vbCrLf & holdText
        MessageBox.Show(message,
            ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub ActItems_Click(sender As Object, e As EventArgs) Handles ActItems.Click
        ' ----- Provide access to patron's circulation records and fees.
        Call (New PatronRecord).ViewPatronRecord(ActiveID, False)
    End Sub

    Private Sub RecordBarcode_Leave(sender As Object, e As EventArgs) Handles RecordBarcode.Leave
        ' ----- Allow an enter to close the form again.
        Me.AcceptButton = ActOK
    End Sub

    Private Sub RecordBarcode_KeyPress(sender As Object, e As KeyPressEventArgs) Handles RecordBarcode.KeyPress
        ' ----- Ignore the Enter key.
        If (e.KeyChar = ChrW(Keys.Return)) Then e.Handled = True
    End Sub
End Class
