﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Public Class ReportSelect
    Private Sub LoadReportGroup(groupNode As XElement, indentLevel As Integer)
        ' ----- Add the groups and items at this level,
        '       and recurse as needed.
        Dim detailNode As XElement
        Dim reportEntry As ReportItem

        ' ----- Process each item or group.
        For Each scanNode As XElement In groupNode.Nodes
            ' ----- Build a content item for the list.
            reportEntry = New ReportItem
            reportEntry.Indent = indentLevel

            ' ----- Get the display name.
            detailNode = scanNode.<displayText>
            If (detailNode Is Nothing) Then Continue For
            reportEntry.DisplayText = detailNode.Value

            If (scanNode.Name = "reportGroup") Then
                ' ----- Start a new display group.
                reportEntry.ItemType = ReportItemEnum.GroupLabel
                AllReports.Items.Add(reportEntry)

                ' ----- Recurse to child items.
                LoadReportGroup(scanNode, indentLevel + 1)
            ElseIf (scanNode.Name = "reportItem") Then
                ' ----- This is an item. Record its location.
                detailNode = scanNode.<reportPath>
                If (detailNode IsNot Nothing) Then _
                   reportEntry.ReportPath = detailNode.Value

                ' ----- Get any command-line arguments.
                detailNode = scanNode.<reportArgs>
                If (detailNode IsNot Nothing) Then _
                   reportEntry.ReportArgs = detailNode.Value

                ' ----- Get any item-specific flags.
                detailNode = scanNode.<reportFlags>
                If (detailNode IsNot Nothing) Then
                    ' ---- "U" adds "-u loginid" to the command.
                    If (detailNode.Value.ToUpper().Contains("U") = True) And
                       (LoggedInUserName.Length <> 0) Then reportEntry.ReportArgs =
                       (reportEntry.ReportArgs & " -u " & LoggedInUserName).Trim()
                End If

                ' ----- Store the full description.
                detailNode = scanNode.<description>
                If (detailNode IsNot Nothing) Then _
                   reportEntry.Description = detailNode.Value

                ' ----- So, what type of entry is it?
                If (scanNode.@type = "built-in") Then
                    ' ----- Built-in program. Check for valid ID.
                    If (IsNumeric(reportEntry.ReportPath) = False) Or
                       (Val(reportEntry.ReportPath) < 1) Or
                       (Val(reportEntry.ReportPath) >
                       CInt(ReportItemEnum.BuiltInStatistics)) Then _
                       Continue For
                    reportEntry.ItemType = CType(CInt(
                       reportEntry.ReportPath), ReportItemEnum)
                    If (reportEntry.Description.Length = 0) Then _
                        reportEntry.Description = GetBuiltinReportDescription(
                        reportEntry.ItemType)
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.@type = "program") Then
                    ' ----- EXE program-based report.
                    If (reportEntry.ReportPath.Length = 0) Then Continue For
                    reportEntry.ItemType = ReportItemEnum.ExeProgram
                    AllReports.Items.Add(reportEntry)
                ElseIf (scanNode.@type = "url") Then
                    ' ----- URL-based report.
                    If (reportEntry.ReportPath.Length = 0) Then Continue For
                    reportEntry.ItemType = ReportItemEnum.UrlProgram
                    AllReports.Items.Add(reportEntry)
                End If
            End If
        Next scanNode
    End Sub

    Private Sub RefreshReportList()
        ' ----- Load in the list of available reports.
        Dim configFile As String
        Dim configData As XDocument
        Dim reportEntry As ReportItem
        Dim counter As Integer

        ' ----- Clear the existing list.
        AllReports.Items.Clear()

        ' ----- Get the location of the configuration file.
        configFile = My.Settings.ReportConfig & ""

        ' ----- Load the configuration file.
        If (configFile.Length <> 0) Then
            If (System.IO.File.Exists(configFile)) Then
                Try
                    ' ----- Load in the file.
                    configData = XDocument.Load(configFile)

                    ' ----- Process the configuration file.
                    LoadReportGroup(configData.FirstNode, 0)
                Catch ex As Exception
                    GeneralError("ReportSelect.RefreshReportList", ex)
                End Try
            End If
        End If

        ' ----- If the configuration file resulted in no reports
        '       appearing in the list, add the default reports.
        If (AllReports.Items.Count = 0) Then
            For counter = 1 To CInt(ReportItemEnum.BuiltInStatistics)
                ' ----- Build the report entry.
                reportEntry = New ReportItem
                reportEntry.Indent = 0
                reportEntry.ItemType = CType(counter, ReportItemEnum)
                Select Case reportEntry.ItemType
                    Case ReportItemEnum.BuiltInCheckedOut
                        reportEntry.DisplayText = "Items Checked Out"
                    Case ReportItemEnum.BuiltInOverdue
                        reportEntry.DisplayText = "Items Overdue"
                    Case ReportItemEnum.BuiltInMissing
                        reportEntry.DisplayText = "Items Missing"
                    Case ReportItemEnum.BuiltInFinesOwed
                        reportEntry.DisplayText = "Patron Fines Owed"
                    Case ReportItemEnum.BuiltInStatistics
                        reportEntry.DisplayText = "Database Statistics"
                End Select
                reportEntry.Description = GetBuiltinReportDescription(
                    reportEntry.ItemType)

                ' ----- Add the report entry to the list.
                AllReports.Items.Add(reportEntry)
            Next counter
        End If
    End Sub

    Private Sub AllReports_SelectedIndexChanged(sender As Object, e As EventArgs) Handles AllReports.SelectedIndexChanged
        ' ----- Display a description of the report, if available.
        Dim reportEntry As Library.ReportItem

        ' ----- Clear any previous description.
        FullDescription.Text = "No report selected."
        If (AllReports.SelectedIndex <> -1) Then
            ' ----- Locate the content and display it.
            reportEntry = DirectCast(AllReports.SelectedItem, Library.ReportItem)
            FullDescription.Text = reportEntry.Description
        End If
    End Sub

    Private Sub ReportSelect_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "ReportSelect.htm")
    End Sub

    Private Sub ReportSelect_Load(sender As Object, e As EventArgs) Handles Me.Load
        ' ----- Display the list of reports.
        RefreshReportList()
    End Sub

    Private Sub ActRun_Click(sender As Object, e As EventArgs) Handles ActRun.Click
        ' ----- Run the selected report.
        Dim reportEntry As Library.ReportItem

        ' ----- Make sure a report is selected.
        If (AllReports.SelectedIndex = -1) Then
            MessageBox.Show("Please select a report from the list.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Different code for each type of entry.
        reportEntry = DirectCast(AllReports.SelectedItem, Library.ReportItem)
        Me.Cursor = Cursors.WaitCursor
        Select Case reportEntry.ItemType
            Case ReportItemEnum.GroupLabel
                ' ----- No report for group entries.
                MessageBox.Show("Please select a report from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Case ReportItemEnum.BuiltInCheckedOut
                ' ----- Items Checked Out
                BasicReportCheckedOut()
            Case ReportItemEnum.BuiltInOverdue
                ' ----- Items Overdue
                BasicReportOverdue()
            Case ReportItemEnum.BuiltInMissing
                ' ----- Items Missing
                BasicReportMissing()
            Case ReportItemEnum.BuiltInFinesOwed
                ' ----- Fines Owed by Patrons
                BasicReportFines()
            Case ReportItemEnum.BuiltInStatistics
                ' ----- Library Database Statistics
                BasicReportStatistics()
            Case ReportItemEnum.ExeProgram
                ' ----- Start a program.
                Try
                    Process.Start("""" & reportEntry.ReportPath & """ " &
                        reportEntry.ReportArgs)
                Catch ex As Exception
                    GeneralError("ReportSelect.ActRun_Click.Exe", ex)
                End Try
            Case ReportItemEnum.UrlProgram
                ' ----- Start a URL.
                Process.Start(reportEntry.ReportPath)
        End Select
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub BasicReportCheckedOut()
        ' ----- Run built-in report #1: Items checked out report.
        Dim sqlText As String
        Dim reportData As DataTable
        Dim reportForm As ReportBuiltinViewer

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " &
            "PA.Barcode AS PatronBarcode, " &
            "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " &
            "NI.Title, CMT.FullName AS MediaName " &
            "FROM Patron AS PA " &
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " &
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " &
            "WHERE PC.Returned = 0 " &
            "AND PC.Missing = 0 " &
            "AND IC.Missing = 0 " &
            "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName"
        Try
            reportData = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportCheckedOut.Retrieve", ex)
            Return
        End Try

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MessageBox.Show("No items are checked out.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        Try
            reportForm.StartReport("Library.ReportCheckedOut.rdlc",
                "Library_ReportSchemaPatronItems", reportData)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportCheckedOut.Start", ex)
        End Try
    End Sub

    Private Sub BasicReportOverdue()
        ' ----- Run built-in report #2: Items overdue report.
        Dim sqlText As String
        Dim reportData As DataTable
        Dim reportForm As ReportBuiltinViewer

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " &
            "PA.Barcode AS PatronBarcode, " &
            "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " &
            "NI.Title, CMT.FullName AS MediaName " &
            "FROM Patron AS PA " &
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " &
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " &
            "WHERE PC.Returned = 0 " &
            "AND PC.Missing = 0 " &
            "AND IC.Missing = 0 " &
            "AND PC.DueDate < GETDATE() " &
            "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName"
        Try
            reportData = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportOverdue.Retrieve", ex)
            Return
        End Try

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MessageBox.Show("No items are overdue.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        Try
            reportForm.StartReport("Library.ReportOverdue.rdlc",
                "Library_ReportSchemaPatronItems", reportData)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportOverdue.Start", ex)
        End Try
    End Sub

    Private Sub BasicReportMissing()
        ' ----- Run built-in report #3: Items missing report.
        Dim sqlText As String
        Dim reportData As DataTable
        Dim reportForm As ReportBuiltinViewer

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " &
            "PA.Barcode AS PatronBarcode, " &
            "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " &
            "NI.Title, CMT.FullName AS MediaName " &
            "FROM Patron AS PA " &
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " &
            "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " &
            "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " &
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " &
            "WHERE PC.Missing = 1 " &
            "OR IC.Missing = 1 " &
            "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName"
        Try
            reportData = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportMissing.Retrieve", ex)
            Return
        End Try

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MessageBox.Show("No items are missing.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        Try
            reportForm.StartReport("Library.ReportMissing.rdlc",
                "Library_ReportSchemaPatronItems", reportData)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportMissing.Start", ex)
        End Try
    End Sub

    Private Sub BasicReportFines()
        ' ----- Run built-in report #4: Fines owed by patrons report.
        Dim sqlText As String
        Dim reportData As DataTable
        Dim reportForm As ReportBuiltinViewer

        ' ----- Retrieve the data as a dataset.
        sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " &
            "PA.Barcode AS PatronBarcode, " &
            "SUM(PC.Fine - PC.Paid) AS FinesDue " &
            "FROM Patron AS PA " &
            "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " &
            "GROUP BY PA.LastName + ', ' + PA.FirstName, PA.Barcode " &
            "HAVING SUM(PC.Fine - PC.Paid) > 0 " &
            "ORDER BY PatronName"
        Try
            reportData = CreateDataTable(sqlText)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportFines.Retrieve", ex)
            Return
        End Try

        ' ----- Check for no data.
        If (reportData.Rows.Count = 0) Then
            reportData.Dispose()
            MessageBox.Show("There are no patrons with unpaid fines.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return
        End If

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        Try
            reportForm.StartReport("Library.ReportPatronFines.rdlc",
                "Library_ReportSchemaPatronFines", reportData)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportFines.Start", ex)
        End Try
    End Sub

    Private Sub BasicReportStatistics()
        ' ----- Run built-in report #5: Library database statistics report.
        Dim sqlText As String
        Dim reportData As Generic.List(Of ReportSchemaStatistics)
        Dim oneEntry As ReportSchemaStatistics
        Dim reportForm As ReportBuiltinViewer
        Dim resultValue As Integer
        Dim counter As Integer
        Const tableSets As String = "Author,Publisher,Subject," &
            "NamedItem,ItemCopy,Patron"
        Const tableTitles As String = "Authors,Publishers,Subject Headings," &
            "Items,Item Copies,Patrons"

        ' ----- Build the report data. It's all counts from different tables.
        reportData = New Generic.List(Of ReportSchemaStatistics)
        For counter = 1 To CountSubStr(tableSets, ",") + 1
            ' ----- Process one table.
            sqlText = "SELECT COUNT(*) FROM " & tableSets.GetSubStr(",", counter)
            Try
                resultValue = DBGetInteger(ExecuteSQLReturn(sqlText))
            Catch ex As Exception
                GeneralError("ReportSelect.BasicReportStatistics." &
                    tableSets.GetSubStr(",", counter), ex)
                Return
            End Try

            ' ----- Add it to the report data.
            oneEntry = New ReportSchemaStatistics
            oneEntry.EntryName = tableTitles.GetSubStr(",", counter)
            oneEntry.EntryValue = resultValue.ToString()
            reportData.Add(oneEntry)
        Next counter

        ' ----- Send the data to the report.
        reportForm = New ReportBuiltinViewer
        Try
            reportForm.StartReport("Library.ReportStatistics.rdlc",
                "Library_ReportSchemaStatistics", reportData)
        Catch ex As Exception
            GeneralError("ReportSelect.BasicReportStatistics.Start", ex)
        End Try
    End Sub

    Private Function GetBuiltinReportDescription(whichReport As ReportItemEnum) As String
        ' ----- Return a predefined description for the built-in reports.
        Select Case whichReport
            Case ReportItemEnum.BuiltInCheckedOut
                Return "Displays all items currently checked out, sorted by name."
            Case ReportItemEnum.BuiltInOverdue
                Return "Displays all overdue items, sorted by name."
            Case ReportItemEnum.BuiltInMissing
                Return "Displays all missing items, sorted by name."
            Case ReportItemEnum.BuiltInFinesOwed
                Return "Displays all unpaid fines owed by patrons, " &
                    "sorted by patron name."
            Case ReportItemEnum.BuiltInStatistics
                Return "Displays some record counts from the Library database."
            Case Else
                Return "There is no description for this report."
        End Select
    End Function
End Class
