﻿' ----- The Library Project
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Option Infer On

Public Class ItemLookup
    ' ----- Class to track the pseudo-links on the single item view.
    Private Class SingleItemLink
        Public LinkType As Library.LookupMethods
        Public LinkID As Long
    End Class

    ' ----- Class to track the history of the views in this form instance.
    Private Class ItemLookupHistory
        Public HistoryDisplay As String
        Public LookupType As Library.LookupMethods
        Public LookupData As String
    End Class

    ' ----- Class to track items in the MatchingItems list.
    Private Class MatchingItemData
        Public ItemID As Long  ' NamedItem.ID
        Public Title As String
        Public Subtitle As String
        Public Author As String
        Public MediaType As String
        Public CallNumber As String

        Public Overrides Function ToString() As String
            ' ----- Build a simple display string.
            If (Subtitle Is Nothing) OrElse (Subtitle.Length = 0) Then
                Return Title & ", by " & Author
            Else
                Return Title & ": " & Subtitle & ", by " & Author
            End If
        End Function
    End Class

    ' ----- Class-level member variables.
    Private UseLC As Boolean
    Private ItemLinkSet As Generic.Dictionary(Of Long, SingleItemLink)
    Private LookupHistorySet As Generic.Stack(Of ItemLookupHistory)
    Private LimitByMedia As Long
    Private LimitByLocation As Long
    Private SecondItemRow As Integer

    Friend Sub InitiateSearch(searchType As Library.LookupMethods,
            searchText As String, limitMedia As Long, limitLocation As Long)
        ' ----- Start a new item search.
        LimitByMedia = limitMedia
        LimitByLocation = limitLocation

        ' ----- Determine if Dewey or LC is used, and other initialization.
        UseLC = CBool(Val(GetSystemValue("UseLC")))
        ItemLinkSet = New Generic.Dictionary(Of Long, SingleItemLink)
        LookupHistorySet = New Generic.Stack(Of ItemLookupHistory)

        ' ----- Perform the first search.
        If (PerformLookup(searchType, searchText, True) = False) Then
            MessageBox.Show("Your search resulted in no matches.",
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information)
            ItemLinkSet = Nothing
            LookupHistorySet = Nothing
            Return
        End If

        ' ----- Add a history item.
        AddLookupHistory(searchType, searchText)

        ' ----- Show the results to the user.
        Me.ShowDialog()
        ItemLinkSet = Nothing
        LookupHistorySet = Nothing
    End Sub

    Private Function HTMLEncode(origText As String) As String
        ' ----- Prepare some text for use in a web page.
        Dim result As String

        result = origText.Replace("&", "&amp;")
        result = result.Replace("<", "&lt;")
        result = result.Replace(">", "&gt;")
        Return result
    End Function

    Private Sub AddLookupHistory(searchType As Library.LookupMethods,
            searchText As String)
        ' ----- Add an item to the lookup history.
        Dim newHistory As ItemLookupHistory
        Dim displayText As String

        ' ----- Build the text for display in the new item.
        displayText = BuildDisplayText(searchType, searchText)

        ' ----- Build the new history item.
        newHistory = New ItemLookupHistory
        newHistory.LookupType = searchType
        newHistory.LookupData = searchText
        newHistory.HistoryDisplay = displayText
        LookupHistorySet.Push(newHistory)

        ' ----- Update the back button.
        RefreshBackButtons()
    End Sub

    Private Function BuildDisplayText(searchType As Library.LookupMethods,
            searchText As String) As String
        ' ----- Add an item to the lookup history.
        Dim displayText As String
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable

        ' ----- Build the text for display in the new item.
        Select Case searchType
            Case LookupMethods.ByTitle
                ' ----- Search by title text.
                displayText = "Titles matching '" & searchText & "'"
            Case LookupMethods.ByAuthor
                ' ----- Search by author name text.
                displayText = "Authors matching '" & searchText & "'"
            Case LookupMethods.BySubject
                ' ----- Search by subject name text.
                displayText = "Subjects matching '" & searchText & "'"
            Case LookupMethods.ByKeywordAny
                ' ----- Search by keyword (any).
                displayText = "Items with any keyword(s): " & searchText
            Case LookupMethods.ByKeywordAll
                ' ----- Search by keyword (all).
                displayText = "Items with all keyword(s): " & searchText
            Case LookupMethods.ByPublisher
                ' ----- Search by publisher name text.
                displayText = "Publishers matching '" & searchText & "'"
            Case LookupMethods.BySeries
                ' ----- Search by series name text.
                displayText = "Series matching '" & searchText & "'"
            Case LookupMethods.BySeriesID
                ' ----- Items for a specific series.
                sqlText = "SELECT FullName FROM CodeSeries WHERE ID = @RecordID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("RecordID", searchText)
                Try
                    displayText = "Items from series '" &
                        DBGetText(ExecuteSQLReturn(sqlRun)) & "'"
                Catch ex As Exception
                    GeneralError("ItemLookup.BuildDisplayText.Series", ex)
                    Return "Error"
                End Try
            Case LookupMethods.ByBarcode
                ' ----- Item with a copy of a specific barcode.
                displayText = "Item with copy barcode '" & searchText & "'"
            Case LookupMethods.ByDatabaseID
                ' ----- Item with a specific ID.
                sqlText = "SELECT Title FROM NamedItem WHERE ID = @RecordID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("RecordID", searchText)
                Try
                    displayText = "Item with title '" &
                        DBGetText(ExecuteSQLReturn(sqlRun)) & "'"
                Catch ex As Exception
                    GeneralError("ItemLookup.BuildDisplayText.Database", ex)
                    Return "Error"
                End Try
            Case LookupMethods.ByAuthorID
                ' ----- Items for a specific author.
                sqlText = "SELECT * FROM Author WHERE ID = @RecordID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("RecordID", searchText)
                Try
                    dbInfo = CreateDataTable(sqlRun)
                Catch ex As Exception
                    GeneralError("ItemLookup.BuildDisplayText.Author", ex)
                    Return "Error"
                End Try
                displayText = "Items by author/name '" &
                    FormatAuthorName(dbInfo.Rows(0)) & "'"
                dbInfo.Dispose()
            Case LookupMethods.BySubjectID
                ' ----- Items for a specific subject.
                sqlText = "SELECT FullName FROM Subject WHERE ID = @RecordID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("RecordID", searchText)
                Try
                    displayText = "Items for subject '" &
                        DBGetText(ExecuteSQLReturn(sqlRun)) & "'"
                Catch ex As Exception
                    GeneralError("ItemLookup.BuildDisplayText.Subject", ex)
                    Return "Error"
                End Try
            Case LookupMethods.ByPublisherID
                ' ----- Items for a specific publisher.
                sqlText = "SELECT FullName FROM Publisher WHERE ID = @RecordID"
                sqlRun = New DataCommand(sqlText)
                sqlRun.AddLong("RecordID", searchText)
                Try
                    displayText = "Items for publisher '" &
                        DBGetText(ExecuteSQLReturn(sqlRun)) & "'"
                Catch ex As Exception
                    GeneralError("ItemLookup.BuildDisplayText.Publisher", ex)
                    Return "Error"
                End Try
            Case Else
                displayText = "Error"
        End Select

        Return displayText
    End Function

    Private Function PerformLookup(searchType As Library.LookupMethods,
            searchText As String, breakOnFailure As Boolean) As Boolean
        ' ----- Perform a search based on the user's request.
        Dim resultType As String
        Dim matches As Integer
        Dim firstName As String
        Dim lastName As String
        Dim holdText As String
        Dim matchEntry As MatchingItemData
        Dim libraryDC As LibraryDataContext
        Dim idQuery As IQueryable(Of Long) = Nothing
        Dim keywordSet As Generic.List(Of String)

        ' ----- Connect to the LINQ engine.
        Me.Cursor = Cursors.WaitCursor
        libraryDC = New LibraryDataContext(My.Settings.DBConnection)

        If (LimitByMedia = -1&) And (LimitByLocation = -1&) Then
            LabelSpecialLimit.Visible = False
        ElseIf (LimitByMedia = -1&) Then
            LabelSpecialLimit.Text = "Limited by location."
            LabelSpecialLimit.Visible = True
        ElseIf (LimitByLocation = -1&) Then
            LabelSpecialLimit.Text = "Limited by media type."
            LabelSpecialLimit.Visible = True
        Else
            LabelSpecialLimit.Text = "Limited by location and media type."
            LabelSpecialLimit.Visible = True
        End If

        ' ----- Different code for each lookup method.
        If (searchType = LookupMethods.ByAuthor) Then
            ' ----- Prepare the query for an author lookup.
            lastName = searchText.GetSubStr(",", 1).Trim()
            firstName = searchText.GetSubStr(",", 2).Trim()
            If (lastName.Contains("*") = False) And
                (lastName.Length <> 0) Then lastName &= "*"
            If (firstName.Contains("*") = False) And
                (firstName.Length <> 0) Then firstName &= "*"
            Dim authorQuery = From au In libraryDC.QAuthors
            If (lastName.Length <> 0) Then _
                authorQuery = From au In authorQuery
                              Where au.LastName Like lastName
            If (firstName.Length <> 0) Then _
                authorQuery = From au In authorQuery
                              Where au.FirstName Like firstName
            authorQuery = From au In authorQuery
                          Order By lastName, firstName
            resultType = "List"

            ' ----- Run the query and store the results.
            matches = 0
            MatchingGeneral.Items.Clear()
            For Each authorItem In authorQuery
                ' ----- Limit to the specified maximum records.
                If (matches >= SearchMatchLimit) Then
                    matches += 1
                    Exit For
                End If

                ' ----- Author list item.
                MatchingGeneral.Items.Add(New ListItemData(
                    authorItem.FormatAuthorName(), authorItem.ID))
                matches += 1
            Next authorItem
        ElseIf (searchType = LookupMethods.BySubject) Then
            ' ----- Prepare the query for a subject lookup.
            holdText = searchText.Trim()
            If (holdText.Contains("*") = False) Then holdText &= "*"
            Dim subjectQuery = From sb In libraryDC.QSubjects
                               Where sb.FullName Like holdText
                               Order By sb.FullName
            resultType = "List"

            ' ----- Run the query and store the results.
            matches = 0
            MatchingGeneral.Items.Clear()
            For Each subjectItem In subjectQuery
                ' ----- Limit to the specified maximum records.
                If (matches >= SearchMatchLimit) Then
                    matches += 1
                    Exit For
                End If

                ' ----- General list item.
                MatchingGeneral.Items.Add(New ListItemData(
                    subjectItem.FullName, subjectItem.ID))
                matches += 1
            Next subjectItem
        ElseIf (searchType = LookupMethods.ByPublisher) Then
            ' ----- Prepare the query for a publisher lookup.
            holdText = searchText.Trim()
            If (holdText.Contains("*") = False) Then holdText &= "*"
            Dim publisherQuery = From pb In libraryDC.QPublishers
                                 Where pb.FullName Like holdText
                                 Order By pb.FullName
            resultType = "List"

            ' ----- Run the query and store the results.
            matches = 0
            MatchingGeneral.Items.Clear()
            For Each publishItem In publisherQuery
                ' ----- Limit to the specified maximum records.
                If (matches >= SearchMatchLimit) Then
                    matches += 1
                    Exit For
                End If

                ' ----- General list item.
                MatchingGeneral.Items.Add(New ListItemData(
                    publishItem.FullName, publishItem.ID))
                matches += 1
            Next publishItem
        ElseIf (searchType = LookupMethods.BySeries) Then
            ' ----- Prepare the query for a series lookup.
            holdText = searchText.Trim()
            If (holdText.Contains("*") = False) Then holdText &= "*"
            Dim seriesQuery = From se In libraryDC.QCodeSeries
                              Where se.FullName Like holdText
                              Order By se.FullName
            resultType = "List"

            ' ----- Run the query and store the results.
            matches = 0
            MatchingGeneral.Items.Clear()
            For Each seriesItem In seriesQuery
                ' ----- Limit to the specified maximum records.
                If (matches >= SearchMatchLimit) Then
                    matches += 1
                    Exit For
                End If

                ' ----- General list item.
                MatchingGeneral.Items.Add(New ListItemData(
                    seriesItem.FullName, seriesItem.ID))
                matches += 1
            Next seriesItem
        ElseIf (searchType = LookupMethods.ByDatabaseID) Or
                (searchType = LookupMethods.ByBarcode) Then
            ' ----- Lookup by specific ID value.
            If (searchType = LookupMethods.ByDatabaseID) Then
                ' ----- Lookup by database ID of the Named Item.
                idQuery = From ni In libraryDC.QNamedItems
                          Where ni.ID = CLng(searchText)
                          Select ni.ID
            Else
                ' ----- Lookup by the barcode of a copy.
                idQuery = From ic In libraryDC.QItemCopies
                          Where ic.Barcode = searchText.Trim
                          Select ID = ic.ItemID
            End If
            resultType = "Single"

            ' ----- Prepare for a single-item display.
            If (idQuery.Count = 0) Then
                If (breakOnFailure = True) Then
                    ' ----- This was the initial search, and it failed.
                    Me.Cursor = Cursors.Default
                    Return False
                Else
                    SingleItemDetail.Text = "Error"
                    ItemLinks.Items.Clear()
                End If
                matches = 0
            Else
                SingleItemDetail.DocumentText =
                    BuildHTMLAndLinks(idQuery.ToArray(0))
                matches = 1
            End If
        Else
            ' ----- Some item-type lookup. Add in the media and
            '       location limits if needed.
            Dim itemQuery = From ni In libraryDC.QNamedItems
            If (LimitByMedia <> -1&) Then
                ' ----- Limit to a specific media type.
                itemQuery = From ni In itemQuery
                            Where ni.MediaType = LimitByMedia
            End If
            If (LimitByLocation <> -1&) Then
                ' ----- Limit to a specific location for item copies.
                itemQuery = From ni In itemQuery
                            Let inLoc =
                            (From ic In ni.QItemCopies
                             Where ic.Location = LimitByLocation)
                            Where inLoc.Count > 0
                            Select ni
            End If

            ' ----- Alter the query based on the lookup type.
            If (searchType = LookupMethods.ByTitle) Then
                ' ----- Perform a title search.
                holdText = searchText.Trim()
                If (holdText.Contains("*") = False) Then holdText &= "*"
                itemQuery = From ni In itemQuery
                            Where ni.Title Like holdText
            ElseIf (searchType = LookupMethods.ByKeywordAny) Then
                ' ----- Perform a keyword (any) search.
                keywordSet = New Generic.List(Of String)
                keywordSet.AddRange(searchText.ToUpper().Split({","}, StringSplitOptions.None))
                itemQuery = From ni In itemQuery
                            Let keySet = (Aggregate ik In ni.QItemKeywords
                            Into Any(keywordSet.Contains(ik.QKeyword.FullName.ToUpper)))
                            Where keySet = True
                            Select ni
            ElseIf (searchType = LookupMethods.ByKeywordAll) Then
                ' ----- Perform a keyword (all) search.
                keywordSet = New Generic.List(Of String)
                keywordSet.AddRange(searchText.ToUpper().Split({","}, StringSplitOptions.None))
                itemQuery = From ni In itemQuery
                            Let keySet = (Aggregate ik In ni.QItemKeywords
                            Into Count(keywordSet.Contains(ik.QKeyword.FullName.ToUpper)))
                            Where keySet = keywordSet.Count
                            Select ni
            ElseIf (searchType = LookupMethods.BySeriesID) Then
                ' ----- Return all items within a series.
                itemQuery = From ni In itemQuery
                            Where ni.Series = CLng(searchText)
            ElseIf (searchType = LookupMethods.ByAuthorID) Then
                ' ----- Return all items by an author.
                itemQuery = From ni In itemQuery
                            Let aiSet =
                            (From au In ni.QItemAuthors
                             Where au.AuthorID = CLng(searchText))
                            Where aiSet.Count > 0
                            Select ni
            ElseIf (searchType = LookupMethods.BySubjectID) Then
                ' ----- Return all items on a certain subject.
                itemQuery = From ni In itemQuery
                            Let subset =
                            (From sb In ni.QItemSubjects
                             Where sb.SubjectID = CLng(searchText))
                            Where subset.Count > 0
                            Select ni
            ElseIf (searchType = LookupMethods.ByPublisherID) Then
                ' ----- Return all items by a publisher.
                itemQuery = From ni In itemQuery
                            Where ni.Publisher = CLng(searchText)
            End If
            resultType = "Item"

            ' ----- All item queries sort by title.
            itemQuery = From ni In itemQuery
                        Order By ni.Title, ni.Subtitle

            ' ----- Prepare for item matches.
            MatchingItems.Items.Clear()
            matches = 0
            For Each itemItem In itemQuery
                ' ----- Limit to the specified maximum records.
                If (matches >= SearchMatchLimit) Then
                    matches += 1
                    Exit For
                End If

                ' ----- Store a new display item.
                matchEntry = New MatchingItemData
                matchEntry.ItemID = itemItem.ID
                matchEntry.Title = itemItem.Title
                matchEntry.Subtitle = itemItem.Subtitle
                If (itemItem.QItemAuthors.Count > 0) Then
                    ' ----- Get the top author in preference order.
                    Dim topAuthor = From au In itemItem.QItemAuthors
                                    Order By au.Sequence
                                    Select au.QAuthor
                    matchEntry.Author = topAuthor(0).FormatAuthorName()
                    If (itemItem.QItemAuthors.Count > 1) Then _
                        matchEntry.Author &= ", et. al."
                End If
                matchEntry.MediaType = itemItem.QCodeMediaType.FullName
                holdText = ""
                If (UseLC = True) Then
                    If (itemItem.LC IsNot Nothing) Then holdText =
                        itemItem.LC.GetSubStr("/", 1)
                Else
                    If (itemItem.Dewey IsNot Nothing) Then holdText =
                        itemItem.Dewey.GetSubStr("/", 1)
                End If
                matchEntry.CallNumber = holdText

                ' ----- Store the match in the list.
                MatchingItems.Items.Add(matchEntry)
                matches += 1
            Next itemItem
        End If

        ' ----- Error if nothing matched the initial search.
        If (matches = 0) And (breakOnFailure = True) Then
            Me.Cursor = Cursors.Default
            Return False
        End If

        ' ----- Update the display.
        PanelMatches.Visible = False
        PanelItems.Visible = False
        PanelOneItem.Visible = False
        If (resultType = "Single") Then
            LabelSingleItem.Text =
                BuildDisplayText(searchType, searchText)
            PanelOneItem.Visible = True
            SingleItemDetail.Focus()
        ElseIf (resultType = "Item") Then
            ' ----- Display the results.
            LabelMatchingItems.Text =
                BuildDisplayText(searchType, searchText)
            If (matches = 1) Then
                LabelItemsCount.Text = "1 matching item found."
            ElseIf (matches > SearchMatchLimit) Then
                LabelItemsCount.Text = "Limited to first " &
                SearchMatchLimit & " matching items."
            Else
                LabelItemsCount.Text = matches & " matching items found."
            End If
            ActItemLookup.Enabled = False
            PanelItems.Visible = True
        Else
            ' ----- Display the results.
            LabelMatchingGeneral.Text =
                BuildDisplayText(searchType, searchText)
            If (matches = 1) Then
                LabelMatchCount.Text = "1 match found."
            ElseIf (matches > SearchMatchLimit) Then
                LabelMatchCount.Text = "Limited to first " &
                SearchMatchLimit & " matches."
            Else
                LabelMatchCount.Text = matches & " matches found."
            End If
            ActGeneralLookup.Enabled = False
            PanelMatches.Visible = True
        End If

        ' ----- Finished.
        libraryDC.Dispose()
        Me.Cursor = Cursors.Default
        Return True
    End Function

    Private Sub RefreshBackButtons()
        ' ----- Refresh the display of the back button items.
        Dim counter As Integer
        Dim scanSet As IEnumerator
        Dim scanHistory As ItemLookupHistory
        Dim whichMenu As ToolStripMenuItem
        Dim useCanvas As Drawing.Graphics = Me.CreateGraphics()

        ' ----- See if there is any history.
        If (LookupHistorySet.Count <= 1) Then
            ' ----- No extra history available.
            ActBack.Enabled = False
            ActBackMenu.Enabled = False
        Else
            ' ----- Build the history. Skip the first item as it is the
            '       one that is displayed.
            counter = 1
            scanSet = LookupHistorySet.GetEnumerator()
            scanSet.MoveNext() ' Skip first entry.
            Do While scanSet.MoveNext()
                scanHistory = DirectCast(scanSet.Current, ItemLookupHistory)
                Select Case counter
                    Case 1 : whichMenu = BackMenu1
                    Case 2 : whichMenu = BackMenu2
                    Case 3 : whichMenu = BackMenu3
                    Case 4 : whichMenu = BackMenu4
                    Case 5 : whichMenu = BackMenu5
                    Case 6 : whichMenu = BackMenu6
                    Case 7 : whichMenu = BackMenu7
                    Case 8 : whichMenu = BackMenu8
                    Case 9 : whichMenu = BackMenu9
                    Case Else : whichMenu = BackMenu10
                End Select

                whichMenu.Text = FitTextToWidth(scanHistory.HistoryDisplay,
                    Me.Width \ 2, useCanvas, whichMenu.Font)
                whichMenu.Visible = True
                counter += 1
                If (counter > 10) Then Exit Do
            Loop

            ' ----- Hide unused menu items.
            Do While (counter <= 10)
                Select Case counter
                    Case 1 : whichMenu = BackMenu1
                    Case 2 : whichMenu = BackMenu2
                    Case 3 : whichMenu = BackMenu3
                    Case 4 : whichMenu = BackMenu4
                    Case 5 : whichMenu = BackMenu5
                    Case 6 : whichMenu = BackMenu6
                    Case 7 : whichMenu = BackMenu7
                    Case 8 : whichMenu = BackMenu8
                    Case 9 : whichMenu = BackMenu9
                    Case Else : whichMenu = BackMenu10
                End Select
                whichMenu.Visible = False
                counter += 1
            Loop
            ActBack.Enabled = True
            ActBackMenu.Enabled = True
        End If
        useCanvas.Dispose()
    End Sub

    Private Function BuildHTMLAndLinks(itemID As Long) As String
        ' ----- Given the ID of a library item, build the HTML text display
        '       that shows all of the relevant detail.
        Dim sqlText As String
        Dim sqlRun As DataCommand
        Dim dbInfo As DataTable
        Dim detailBody As New Text.StringBuilder
        Dim counter As Integer
        Dim oneRow As DataRow
        Dim holdText As String
        Dim entryID As Long
        Dim newLink As SingleItemLink

        Me.Cursor = Cursors.WaitCursor

        ' ----- Initialize.
        entryID = 0&
        ItemLinkSet.Clear()
        ItemLinks.Items.Clear()

        ' ----- The main HTML headers are stored in a text file resource.
        '       Add it to the body content now.
        detailBody.Append(My.Resources.ItemLookupBody)

        ' ----- Add in the title.
        sqlText = "SELECT Title, Subtitle FROM NamedItem WHERE ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("ItemLookup.BuildHTMLAndLinks.Title", ex)
            Return "Error"
        End Try
        oneRow = dbInfo.Rows(0)
        detailBody.AppendLine("<h1>" & HTMLEncode(DBGetText(oneRow!Title)) & "</h1>")

        ' ----- Add in the subtitle if available.
        If (IsDBNull(oneRow!Subtitle) = False) Then
            detailBody.AppendLine("<h3>" & HTMLEncode(DBGetText(oneRow!Subtitle)) & "</h3>")
        End If
        dbInfo.Dispose()

        ' ----- Add in the authors and names.
        sqlText = "SELECT AU.*, IA.Sequence, CA.FullName FROM ItemAuthor AS IA " &
            "INNER JOIN Author AS AU ON IA.AuthorID = AU.ID INNER JOIN " &
            "CodeAuthorType AS CA ON IA.AuthorType = CA.ID " &
            "WHERE IA.ItemID = @RecordID ORDER BY IA.Sequence"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("ItemLookup.BuildHTMLAndLinks.Authors", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then
            ' ----- There are authors or names. Display a heading.
            detailBody.AppendLine("<h2>Authors and Names</h2>")
            For Each dbRow As DataRow In dbInfo.Rows
                ' ----- Add in this one author name.
                holdText = dbRow.FormatAuthorName()
                entryID += 1
                detailBody.AppendLine("<p><a href=""library://" & entryID & """>" &
                    HTMLEncode(holdText & " [" & DBGetText(dbRow!FullName) & "]") &
                    "</a></p>")

                ' ----- Add in an author link.
                newLink = New SingleItemLink
                newLink.LinkType = General.LookupMethods.ByAuthorID
                newLink.LinkID = DBGetLong(dbRow!ID)
                ItemLinkSet.Add(entryID, newLink)
                ItemLinks.Items.Add(New ListItemData("Author: " &
                    holdText & " [" & DBGetText(dbRow!FullName) & "]", entryID))
            Next dbRow
        End If
        dbInfo.Dispose()

        ' ----- Retrieve most of the remaining fields.
        sqlText = "SELECT NI.Dewey, NI.Dewey, NI.LC, NI.Description, NI.Publisher, " &
            "PB.FullName AS PubName, NI.Edition, NI.Copyright, NI.ISxN, NI.Series, " &
            "CS.FullName AS SeriesName, CMT.FullName AS MediaName " &
            "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " &
            "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " &
            "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " &
            "WHERE NI.ID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("ItemLookup.BuildHTMLAndLinks.Fields", ex)
            Return "Error"
        End Try
        oneRow = dbInfo.Rows(0)

        ' ----- Add in the call number information.
        detailBody.AppendLine("<h2>Call Number</h2>")
        holdText = "No call number available."
        If (UseLC) Then
            ' ----- Show the Library of Congress number.
            If (IsDBNull(oneRow!LC) = False) Then holdText = DBGetText(oneRow!LC)
        Else
            ' ----- Show the Dewey Decimal number.
            If (IsDBNull(oneRow!Dewey) = False) Then holdText = DBGetText(oneRow!Dewey)
        End If
        For counter = 1 To CountSubStr(holdText, "/") + 1
            ' ----- Add one line of the number.
            detailBody.AppendLine("<p>" & HTMLEncode(
                holdText.GetSubStr("/", counter).Trim()) & "</p>")
        Next counter

        ' ----- Display the media type name.
        detailBody.AppendLine("<p><i>" & HTMLEncode(DBGetText(oneRow!MediaName)) &
            "</i></p>")

        ' ----- Add in the item description.
        If (IsDBNull(oneRow!Description) = False) Then
            detailBody.AppendLine("<h2>Description</h2>")
            detailBody.AppendLine("<p>" & HTMLEncode(DBGetText(oneRow!Description)) & "</p>")
        End If

        ' ----- Add in the publishing information.
        counter = 0
        If (IsDBNull(oneRow!PubName) = False) Then counter += 1
        If (IsDBNull(oneRow!Edition) = False) Then counter += 1
        If (IsDBNull(oneRow!Copyright) = False) Then counter += 1
        If (IsDBNull(oneRow!ISxN) = False) Then counter += 1
        If (counter > 0) Then
            ' ----- There is some publishing information to display.
            detailBody.AppendLine("<h2>Publishing Information</h2>")

            ' ----- Add the publisher name, if available.
            If (IsDBNull(oneRow!PubName) = False) Then
                entryID += 1
                holdText = DBGetText(oneRow!PubName)
                detailBody.AppendLine("<p><a href=""library://" & entryID & """>" &
                    HTMLEncode(holdText) & "</a></p>")

                ' ----- Add in a publisher link.
                newLink = New SingleItemLink
                newLink.LinkType = General.LookupMethods.ByPublisherID
                newLink.LinkID = DBGetLong(oneRow!Publisher)
                ItemLinkSet.Add(entryID, newLink)
                ItemLinks.Items.Add(New ListItemData("Publisher: " &
                    holdText, entryID))
            End If

            ' ----- Add the edition number, if available.
            If (IsDBNull(oneRow!Edition) = False) Then
                detailBody.AppendLine("<p>Edition: " &
                    HTMLEncode(DBGetText(oneRow!Edition)) & "</p>")
            End If

            ' ----- Add the copyright year, if available.
            If (IsDBNull(oneRow!Copyright) = False) Then
                detailBody.AppendLine("<p>Copyright: " &
                    DBGetInteger(oneRow!Copyright).ToString() & "</p>")
            End If

            ' ----- Add the ISBN/ISSN number, if available.
            If (IsDBNull(oneRow!ISxN) = False) Then
                detailBody.AppendLine("<p>ISBN / ISSN: " &
                    HTMLEncode(DBGetText(oneRow!ISxN)) & "</p>")
            End If
        End If

        ' ----- Add in the series information.
        If (IsDBNull(oneRow!SeriesName) = False) Then
            detailBody.AppendLine("<h2>Related Series</h2>")

            entryID += 1
            holdText = DBGetText(oneRow!SeriesName)
            detailBody.AppendLine("<p><a href=""library://" & entryID & """>" &
                HTMLEncode(holdText) & "</a></p>")

            ' ----- Add in a series link.
            newLink = New SingleItemLink
            newLink.LinkType = General.LookupMethods.BySeriesID
            newLink.LinkID = DBGetLong(oneRow!Series)
            ItemLinkSet.Add(entryID, newLink)
            ItemLinks.Items.Add(New ListItemData("Series: " & holdText, entryID))
        End If
        dbInfo.Dispose()

        ' ----- Add in the related subjects.
        sqlText = "SELECT SB.* FROM ItemSubject AS IB INNER JOIN Subject AS SB " &
            "ON IB.SubjectID = SB.ID WHERE IB.ItemID = @RecordID " &
            "ORDER BY SB.FullName"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemID)
        Try
            dbInfo = CreateDataTable(sqlRun)
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("ItemLookup.BuildHTMLAndLinks.Subjects", ex)
            Return "Error"
        End Try
        If (dbInfo.Rows.Count > 0) Then
            ' ----- There are subjects. Display a heading.
            detailBody.AppendLine("<h2>Related Subjects</h2>")

            For Each dbRow As DataRow In dbInfo.Rows
                ' ----- Add in this one subject name.
                entryID += 1
                holdText = DBGetText(dbRow!FullName)
                detailBody.AppendLine("<p><a href=""library://" & entryID & """>" &
                    HTMLEncode(holdText) & "</a></p>")

                ' ----- Add in a subject link.
                newLink = New SingleItemLink
                newLink.LinkType = General.LookupMethods.BySubjectID
                newLink.LinkID = DBGetLong(dbRow!ID)
                ItemLinkSet.Add(entryID, newLink)
                ItemLinks.Items.Add(New ListItemData("Subject: " & holdText, entryID))
            Next dbRow
        End If
        dbInfo.Dispose()

        ' ----- See how many copies there are.
        sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = @RecordID"
        sqlRun = New DataCommand(sqlText)
        sqlRun.AddLong("RecordID", itemID)
        Try
            counter = DBGetInteger(ExecuteSQLReturn(sqlRun))
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            GeneralError("ItemLookup.BuildHTMLAndLinks.Copies", ex)
            Return "Error"
        End Try

        ' ----- Add in the copies information if needed.
        If (counter > 0) Then
            ' ----- Introduce the copies.
            If (counter = 1) Then holdText = "1 copy" Else holdText = counter & " copies"
            detailBody.AppendLine("<h2>Copies</h2>")
            detailBody.AppendLine("<p>" & holdText & "</p>")

            ' ----- Build a table that lists the copies. Use a data table
            '       since the call to GetCopyDisposition() will open
            '       a separate data reader.
            detailBody.AppendLine("<table cellspacing=""0"" cellpadding=""3"">")
            detailBody.AppendLine("<tr><th>Copy</th><th>Location</th>" &
                "<th>Status</th></tr>")
            sqlText = "SELECT IC.ID, IC.CopyNumber, CL.FullName FROM ItemCopy AS IC " &
                "LEFT JOIN CodeLocation AS CL ON IC.Location = CL.ID " &
                "WHERE IC.ItemID = @RecordID ORDER BY IC.CopyNumber"
            sqlRun = New DataCommand(sqlText)
            sqlRun.AddLong("RecordID", itemID)
            Try
                dbInfo = CreateDataTable(sqlRun)
            Catch ex As Exception
                Me.Cursor = Cursors.Default
                GeneralError("ItemLookup.BuildHTMLAndLinks.OneCopy", ex)
                Return "Error"
            End Try
            For Each dbRow As DataRow In dbInfo.Rows
                ' ----- Add the copy number.
                detailBody.Append("<tr><td>" & DBGetInteger(dbRow!CopyNumber).ToString() & "</td>")

                ' ----- Add the location.
                If (IsDBNull(dbRow!FullName) = True) Then holdText = "Not listed" _
                    Else holdText = HTMLEncode(DBGetText(dbRow!FullName))
                detailBody.Append("<td>" & holdText & "</td>")

                ' ----- Add the disposition.
                detailBody.AppendLine("<td>" & HTMLEncode(GetCopyDisposition(
                    DBGetLong(dbRow!ID))) & "</td></tr>")
            Next dbRow
            dbInfo.Dispose()
            dbInfo = Nothing
            detailBody.AppendLine("</table>")
        End If

        ' ----- Finished.
        detailBody.AppendLine("</body>")
        detailBody.AppendLine("</html>")
        Return detailBody.ToString()
    End Function

    Private Sub MatchingGeneral_DoubleClick(sender As Object, e As EventArgs) Handles MatchingGeneral.DoubleClick
        ' ----- Same as a click on the Lookup button.
        ActGeneralLookup.PerformClick()
    End Sub

    Private Sub ActGeneralLookup_Click(sender As Object, e As EventArgs) Handles ActGeneralLookup.Click
        ' ----- Look up the items associated with the selected match.
        Dim baseType As Library.LookupMethods
        Dim searchType As Library.LookupMethods
        Dim itemID As Long

        ' ----- Ignore if no match is selected.
        If (MatchingGeneral.SelectedIndex = -1) Then Return
        itemID = DirectCast(MatchingGeneral.SelectedItem, ListItemData).ItemData

        ' ----- Determine the type of search.
        baseType = LookupHistorySet.Peek.LookupType
        Select Case baseType
            Case LookupMethods.ByAuthor
                searchType = LookupMethods.ByAuthorID
            Case LookupMethods.BySubject
                searchType = LookupMethods.BySubjectID
            Case LookupMethods.ByPublisher
                searchType = LookupMethods.ByPublisherID
            Case LookupMethods.BySeries
                searchType = LookupMethods.BySeriesID
        End Select

        ' ----- Perform the lookup.
        If (PerformLookup(searchType, itemID.ToString(), False) = False) Then Return

        ' ----- Store the history.
        AddLookupHistory(searchType, itemID.ToString())
    End Sub

    Private Sub MatchingItems_DoubleClick(sender As Object, e As EventArgs) Handles MatchingItems.DoubleClick
        ' ----- Same as the lookup button.
        ActItemLookup.PerformClick()
    End Sub

    Private Sub ActItemLookup_Click(sender As Object, e As EventArgs) Handles ActItemLookup.Click
        ' ----- Look up the item with the selected ID.
        Dim itemID As Long

        ' ----- Ignore if no match is selected.
        If (MatchingItems.SelectedIndex = -1) Then Return
        itemID = DirectCast(MatchingItems.SelectedItem, MatchingItemData).ItemID

        ' ----- Perform the lookup.
        If (PerformLookup(LookupMethods.ByDatabaseID, itemID.ToString(), False) = False) _
            Then Return

        ' ----- Store the history.
        AddLookupHistory(LookupMethods.ByDatabaseID, itemID.ToString())
    End Sub

    Private Sub SingleItemDetail_Navigating(sender As Object, e As WebBrowserNavigatingEventArgs) Handles SingleItemDetail.Navigating
        ' ----- Follow the clicked link.
        If (e.Url.Scheme = "library") Then _
            FollowItemLink(CLng(e.Url.Host()))
    End Sub

    Private Sub BackMenuItems_Click(sender As Object, e As EventArgs) _
            Handles BackMenu1.Click, BackMenu2.Click, BackMenu3.Click, BackMenu4.Click,
            BackMenu5.Click, BackMenu6.Click, BackMenu7.Click, BackMenu8.Click,
            BackMenu9.Click, BackMenu10.Click
        ' ----- One of the back menu items was clicked.
        Dim whichItem As Long
        Dim scanHistory As ItemLookupHistory

        ' ----- Determine the clicked item.
        whichItem = CLng(DigitsOnly(DirectCast(sender, ToolStripMenuItem).Name))
        If (whichItem >= LookupHistorySet.Count) Then Return

        ' ----- Get rid of the in-between items.
        For counter As Long = 1& To whichItem
            LookupHistorySet.Pop()
        Next counter

        ' ----- Perform a lookup as requested.
        scanHistory = LookupHistorySet.Peek
        If (PerformLookup(scanHistory.LookupType, scanHistory.LookupData,
            False) = False) Then Return
        RefreshBackButtons()
    End Sub

    Private Sub ActBackMenu_Click(sender As Object, e As EventArgs) Handles ActBackMenu.Click
        ' ----- Show the popup menu for back items.
        BackMenu.Show(ActBackMenu, New Point(-ActBack.Width, ActBackMenu.Height))
    End Sub

    Private Sub ActBack_Click(sender As Object, e As EventArgs) Handles ActBack.Click
        ' ----- Same as a click on the first menu item.
        If (LookupHistorySet.Count > 1) Then BackMenu1.PerformClick()
    End Sub

    Private Sub MatchingGeneral_SelectedIndexChanged(sender As Object, e As EventArgs) Handles MatchingGeneral.SelectedIndexChanged
        ' ----- Activate the lookup button if needed.
        ActGeneralLookup.Enabled = CBool(MatchingGeneral.SelectedIndex <> -1)
    End Sub

    Private Sub MatchingGeneral_Click(sender As Object, e As EventArgs) Handles MatchingGeneral.Click
        ' ----- Activate the lookup button if needed.
        ActGeneralLookup.Enabled = CBool(MatchingGeneral.SelectedIndex <> -1)
    End Sub

    Private Sub MatchingItems_SelectedIndexChanged(sender As Object, e As EventArgs) Handles MatchingItems.SelectedIndexChanged
        ' ----- Activate the lookup button if needed.
        ActItemLookup.Enabled = CBool(MatchingItems.SelectedIndex <> -1)
    End Sub

    Private Sub MatchingItems_Click(sender As Object, e As EventArgs) Handles MatchingItems.Click
        ' ----- Activate the lookup button if needed.
        ActItemLookup.Enabled = CBool(MatchingItems.SelectedIndex <> -1)
    End Sub

    Private Sub ActItemLinks_Click(sender As Object, e As EventArgs) Handles ActItemLinks.Click
        ' ----- Jump to the link indicated in the drop-down list.
        Dim entryLink As Long

        ' ----- Ignore if no link is selected.
        If (ItemLinks.SelectedIndex = -1) Then Return
        entryLink = DirectCast(ItemLinks.SelectedItem, ListItemData).ItemData
        FollowItemLink(entryLink)
    End Sub

    Private Sub FollowItemLink(entryID As Long)
        ' ----- Given a character position in the single item text panel,
        '       follow the link indicated by that item.
        Dim scanLink As SingleItemLink

        ' ----- Access the link.
        scanLink = ItemLinkSet.Item(entryID)
        If (scanLink Is Nothing) Then Return

        ' ----- Perform a lookup as requested.
        If (PerformLookup(scanLink.LinkType, scanLink.LinkID.ToString(), False) = False) _
            Then Return

        ' ----- Store the history.
        AddLookupHistory(scanLink.LinkType, scanLink.LinkID.ToString())
    End Sub

    Private Sub ItemLookup_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        ' ----- F1 shows online help.
        If (e.KeyCode = Keys.F1) Then OnlineHelp(Me, "ItemLookup.htm")
    End Sub

    Private Sub ItemLookup_KeyPress(sender As Object, e As KeyPressEventArgs) Handles MyBase.KeyPress
        ' ----- Treat the backspace key like a click on the Back button.
        If (e.KeyChar = Chr(Keys.Back)) Then
            If (ActBack.Enabled = True) Then ActBack.PerformClick()
        End If
    End Sub

    Private Sub ItemLookup_Load(sender As Object, e As EventArgs) Handles Me.Load
        ' ----- Prepare the form.
        Dim formGraphics As Drawing.Graphics

        ' ----- Set the default height of items in the matching items listbox.
        formGraphics = Me.CreateGraphics()
        MatchingItems.ItemHeight = CInt(formGraphics.MeasureString("A" & vbCrLf & "g",
            MatchingItems.Font).Height()) + 3
        SecondItemRow = CInt(formGraphics.MeasureString("Ag",
            MatchingItems.Font).Height()) + 1
        formGraphics.Dispose()
    End Sub

    Private Sub MatchingItems_DrawItem(sender As Object, e As DrawItemEventArgs) Handles MatchingItems.DrawItem
        ' ----- Draw the matching items on two lines.
        Dim itemToDraw As MatchingItemData
        Dim useBrush As Drawing.Brush
        Dim boldFont As Drawing.Font
        Dim ellipsesText As StringFormat

        ' ----- Draw the background of the item.
        If (CBool(CInt(e.State) And CInt(DrawItemState.Selected))) _
            Then useBrush = SystemBrushes.HighlightText _
            Else useBrush = SystemBrushes.WindowText
        e.DrawBackground()

        ' ----- The title will use a bold version of the main font.
        boldFont = New Drawing.Font(e.Font, FontStyle.Bold)

        ' ----- Obtain the item to draw.
        itemToDraw = DirectCast(MatchingItems.Items(e.Index), MatchingItemData)
        ellipsesText = New StringFormat
        ellipsesText.Trimming = StringTrimming.EllipsisCharacter

        ' ----- Draw the text of the item.
        e.Graphics.DrawString(itemToDraw.Title, boldFont, useBrush,
            New Rectangle(0, e.Bounds.Top, ItemColEnd.Left - MatchingItems.Left,
            boldFont.Height), ellipsesText)
        e.Graphics.DrawString(itemToDraw.Author, e.Font, useBrush,
            New Rectangle(ItemColAuthor.Left, e.Bounds.Top + SecondItemRow,
            ItemColCall.Left - ItemColAuthor.Left - 8, e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemToDraw.CallNumber, e.Font, useBrush,
            New Rectangle(ItemColCall.Left, e.Bounds.Top + SecondItemRow,
            ItemColEnd.Left - ItemColType.Left, e.Font.Height), ellipsesText)
        e.Graphics.DrawString(itemToDraw.MediaType, e.Font, useBrush,
            New Rectangle(ItemColType.Left, e.Bounds.Top + SecondItemRow,
            ItemColType.Left - ItemColCall.Left - 8, e.Font.Height), ellipsesText)

        ' ----- If the ListBox has focus, draw a focus rectangle.
        e.DrawFocusRectangle()
        boldFont.Dispose()
    End Sub
End Class
