﻿' ----- The Library Project: License Generator
'       Start-to-Finish Visual Basic 2015
'       Copyright (c) 2015 by Tim Patrick

Imports System.Xml
Imports System.Security.Cryptography
Imports System.Security.Cryptography.Xml
Imports System.ComponentModel

Class MainWindow
    Private ActiveStatus As KeyFolderStatus

    Private Sub ActGenerate_Click(sender As Object, e As RoutedEventArgs) Handles ActGenerate.Click
        ' ----- Generate the license key.
        Dim licenseXMLNew As XDocument
        Dim licenseXML As XmlDocument
        Dim licenseSaveLocation As Microsoft.Win32.SaveFileDialog

        ' ----- Make sure we have valid data for generation.
        If (VerifyFormData() = False) Then Return

        ' ----- Generate the XML content.
        licenseXMLNew = BuildXmlLicenseContent()
        If (licenseXMLNew Is Nothing) Then Return

        ' ----- Unfortunately, XDocuments, i.e., "the new way," do not offer
        '       an easy way to digitally sign the content. This means we
        '       need to convert it to "the old way."
        licenseXML = New XmlDocument()
        licenseXML.LoadXml(licenseXMLNew.ToString())

        ' ----- Digitally sign the file.
        If (SignXmlLicenseContent(licenseXML) = False) Then Return

        ' ----- Prompt the user for the output location.
        licenseSaveLocation = New Microsoft.Win32.SaveFileDialog()
        licenseSaveLocation.FileName = "LibraryLicense.lic"
        licenseSaveLocation.Filter = "License Files|*.lic|XML Files|*.xml|All Files|*.*"
        licenseSaveLocation.Title = "Save License File"
        If (licenseSaveLocation.ShowDialog() = False) Then Return

        ' ----- Delete any existing license file.
        If (System.IO.File.Exists(licenseSaveLocation.FileName) = True) Then
            Try
                System.IO.File.Delete(licenseSaveLocation.FileName)
            Catch ex As Exception
                MessageBox.Show("The license could not be saved because an existing " &
                    "license file with the same name could not be replaced.",
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
                Return
            End Try
        End If

        ' ----- Save the license to the file.
        Try
            licenseXML.Save(licenseSaveLocation.FileName)
        Catch ex As Exception
            MessageBox.Show("The license could not be saved due to the following error:" &
                vbCrLf & vbCrLf & ex.Message,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return
        End Try

        ' ----- Finished.
        MessageBox.Show("License file saved successfully.",
            ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Information)
        LicenseeName.Focus()
    End Sub

    Private Sub ActLocation_Click(sender As Object, e As RoutedEventArgs) Handles ActLocation.Click
        ' ----- Prompt the user to alter the key pair.
        Dim useFolder As String
        Dim result As Boolean

        ' ----- Promt the user.
        useFolder = KeyLocation.Text
        result = (New KeyLocationWindow).GetKeyFolder(useFolder)
        If (result = True) Then
            ' ----- Save the changes.
            My.Settings.PathForKeyPair = useFolder
            ActiveStatus = GetFolderStatus(useFolder)

            ' ----- Update the display.
            KeyLocation.Text = useFolder
            If (ActiveStatus = KeyFolderStatus.ValidFolder) Then
                StatusDetailColor.Fill = Brushes.Green
            Else
                StatusDetailColor.Fill = Brushes.Red
            End If
            StatusDetailText.Content = GetFolderStatusText(ActiveStatus)
        End If
    End Sub

    Private Sub MainWindow_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded
        ' ----- Prepare the form.
        Dim useFolder As String

        ' ----- Licensing defaults to today.
        LicenseDate.SelectedDate = Today

        ' ----- Display the folder with the key.
        useFolder = My.Settings.PathForKeyPair
        KeyLocation.Text = useFolder
        ActiveStatus = GetFolderStatus(useFolder)
        If (ActiveStatus = KeyFolderStatus.ValidFolder) Then
            StatusDetailColor.Fill = Brushes.Green
        Else
            StatusDetailColor.Fill = Brushes.Red
        End If
        StatusDetailText.Content = GetFolderStatusText(ActiveStatus)

        ' ----- If the keys are not valid, prompt immediately.
        If (ActiveStatus <> KeyFolderStatus.ValidFolder) Then
            ActLocation.RaiseEvent(New RoutedEventArgs(Controls.Primitives.ButtonBase.ClickEvent))
        End If
    End Sub

    Private Function VerifyFormData() As Boolean
        ' ----- Confirm that all form data is valid. Return True on success.
        Dim versionParts() As String
        Dim scanVersion As String
        Const VersionMessage As String =
            "The Covered Version must contain between two and four " &
            "dot-delimited components, and each component must be a " &
            "number between 0 and 9999, or the '*' character."

        ' ----- The folder status must be valid.
        If (ActiveStatus <> KeyFolderStatus.ValidFolder) Then
            MessageBox.Show("The public-private key location and status must both be valid.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return False
        End If

        ' ----- The version number must contain between two and four
        '       dot-delimited parts, and the parts must be numbers
        '       (0 through 9999) or "*".
        versionParts = CoveredVersion.Text.Trim().Split({"."}, StringSplitOptions.None)
        If (versionParts.Length < 2) Or (versionParts.Length > 4) Then
            MessageBox.Show(VersionMessage,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            CoveredVersion.Focus()
            Return False
        End If
        For Each scanVersion In versionParts
            If (IsNumeric(scanVersion) = True) Then
                If (Val(scanVersion) < 0) Or (Val(scanVersion) > 9999) Then
                    MessageBox.Show(VersionMessage,
                        ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
                    CoveredVersion.Focus()
                    Return False
                End If
            ElseIf (scanVersion <> "*") Then
                MessageBox.Show(VersionMessage,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
                CoveredVersion.Focus()
                Return False
            End If
        Next scanVersion

        ' ----- Licensee Name is required.
        If (LicenseeName.Text.Trim().Length = 0) Then
            MessageBox.Show("Licensee Name is required.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            LicenseeName.Focus()
            Return False
        End If

        ' ----- Serial Number is required.
        If (SerialNumber.Text.Trim().Length = 0) Then
            MessageBox.Show("Serial Number is required.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            SerialNumber.Focus()
            Return False
        End If

        ' ----- Success.
        Return True
    End Function

    Private Function BuildXmlLicenseContent() As XDocument
        ' ----- Create the XML license content from the user-supplied data.
        '       The final XML content will look something like this.
        '         <?xml version="1.0" encoding="utf-8"?>
        '         <License>
        '           <Product>Library Project</Product>
        '           <LicenseDate>1/1/2007</LicenseDate>
        '           <ExpireDate>12/31/2999</ExpireDate>
        '           <CoveredVersion>1.*</CoveredVersion>
        '           <Licensee>Your Name Here</Licensee>
        '           <SerialNumber>LIB-123456789</SerialNumber>
        '         </License>
        Return New XDocument(
            New XDeclaration("1.0", "utf-8", "yes"),
            New XElement("License",
                New XElement("Product", ApplicationProductName),
                New XElement("LicenseDate", LicenseDate.SelectedDate.Value.ToString("M/d/yyyy")),
                New XElement("ExpireDate", ExpirationDate.SelectedDate.Value.ToString("M/d/yyyy")),
                New XElement("CoveredVersion", CoveredVersion.Text.Trim()),
                New XElement("Licensee", LicenseeName.Text.Trim()),
                New XElement("SerialNumber", SerialNumber.Text.Trim())
            )
        )
    End Function

    Private Function SignXmlLicenseContent(sourceXML As XmlDocument) As Boolean
        ' ----- Add a digital signature to an XML document.
        Dim privateKeyFile As String
        Dim privateKey As RSA
        Dim signature As SignedXml
        Dim referenceMethod As Reference

        ' ----- Locate the private key file.
        privateKeyFile = System.IO.Path.Combine(KeyLocation.Text, PrivateKeyFilename)
        If (System.IO.File.Exists(privateKeyFile) = False) Then
            MessageBox.Show("Could not locate the private key file.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return False
        End If

        Try
            ' ----- Load in the private key.
            privateKey = RSA.Create()
            privateKey.FromXmlString(System.IO.File.ReadAllText(privateKeyFile))

            ' ----- Create the object that generates the signature.
            signature = New SignedXml(sourceXML)
            signature.SignedInfo.CanonicalizationMethod =
                SignedXml.XmlDsigCanonicalizationUrl
            signature.SigningKey = privateKey

            ' ----- The signature will appear as a <reference> element in the
            '       XML. The signature object can generate that automatically
            '       if we tell it to. It's more important later when trying to
            '       verify the signature. This transform says to ingore the
            '       signature itself when comparing the signature.
            referenceMethod = New Reference("")
            referenceMethod.AddTransform(New XmlDsigEnvelopedSignatureTransform(False))
            signature.AddReference(referenceMethod)

            ' ----- Add the signature to the XML content.
            signature.ComputeSignature()
            sourceXML.DocumentElement.AppendChild(signature.GetXml())
        Catch ex As Exception
            MessageBox.Show("The license could not be digitally signed due to the following " &
                "error:" & vbCrLf & vbCrLf & ex.Message,
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation)
            Return False
        End Try

        ' ----- Finished.
        Return True
    End Function

    Private Sub TextFields_GotFocus(sender As Object, e As RoutedEventArgs) _
            Handles CoveredVersion.GotFocus, LicenseeName.GotFocus, SerialNumber.GotFocus
        ' ----- Highlight the entire text.
        DirectCast(sender, TextBox).SelectAll()
    End Sub

    Private Sub MainWindow_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        ' ----- Save settings changes.
        My.Settings.Save()
    End Sub
End Class
